#!/usr/bin/ruby
# frozen_string_literal: true

raise NotImplementedError

__END__
require 'logger'
require 'jibril'

begin
  $l = Logger.new(STDOUT)
  conf = Jibril::Config.new(
rescue => e
  pp e
end

__END__
sockets = Socket.tcp_server_sockets('0.0.0.0', Jibril::PORT)
sockets.each do |socket|
  $l.info { "Ready to accept: #{socket.local_address.inspect_sockaddr}" }
end

def handle_accept socket
  $l.info { "Handle accept" }

  begin
    con,_ = socket.accept_nonblock
  rescue IO::WaitReadable
    $l.warn { 'Failed to accept!' }
    return
  end

  cert_store = OpenSSL::X509::Store.new
  cert_store.set_default_paths
  cert_store.verify_callback = lambda do |preverify_ok, store_ctx|
    puts "Store callback, #{store_ctx}"
    true
  end

  ctx = OpenSSL::SSL::SSLContext.new
  ctx.key = OpenSSL::PKey::RSA.new(File.open('test/certs/server.key'))
  ctx.cert = OpenSSL::X509::Certificate.new(File.open('test/certs/server.crt'))
  ctx.min_version = :TLS1_2
  ctx.verify_mode = OpenSSL::SSL::VERIFY_PEER | OpenSSL::SSL::VERIFY_FAIL_IF_NO_PEER_CERT
  ctx.cert_store = cert_store
  ctx.verify_callback = lambda do |preverify_ok, store_ctx|
    puts "CTX   callback, #{store_ctx}"
    true
  end

  $l.debug { "Server cert: #{OpenSSL::Digest::MD5.new(ctx.cert.to_der).to_s}" }

  ssl_socket = OpenSSL::SSL::SSLSocket.new(con, ctx)
  ssl_socket.sync_close = true

  begin
    #ssl_socket.accept_nonblock
    ssl_socket.accept
  rescue IO::WaitReadable
    IO.select([con])
    retry
  rescue IO::WaitWritable
    IO.select([con])
    retry
  rescue => e
    $l.warn { "Unexpected exception: #{e}" }
    raise
  end
  $l.info { "Accepted" }

  pp ssl_socket.peer_cert

  begin
    pp ssl_socket.read_nonblock(1024)
  rescue IO::WaitReadable
    IO.select([con])
    retry
  rescue IO::WaitWritable
    IO.select([con])
    retry
  end

  ssl_socket.close

  $l.info { 'SSL close' }
end

while true
  read, _, _ = IO.select(sockets)

  if !read.empty?
    $l.info { "Read ready on server socket" }
    read.each { |socket| handle_accept(socket) }
  end
end

__END__
module Jibril
  class Server
    def initialize(host = '0.0.0.0', port = 9876)
      @threads = []
      @host = host
      @port = port

      @servers = Socket.tcp_server_sockets(@host, @port)
      @servers.each do |s|
        $l.info { "Ready to accept: #{s.local_address.inspect_sockaddr}" }
      end
    end
    def start!
      Socket.accept_loop(@servers) do |con|
        $l.info { "Connection from: #{con.remote_address.inspect_sockaddr}" }
        con.close
      end
    end
  end
end

Jibril::Server.new.start!
